<?php

// PHPOF.conf.php - global default configuration options
// $Id: PHPOF.conf-defaults.php 1424 2008-12-29 16:09:36Z tim $
// You can override any of these options on a per-site basis


/////////////////////////////////////////////////////////////
// BASIC STUFF

// Site name (used in error reports etc.)
$PHPOF_SITE_NAME = '[Un-named; see PHPOF_SITE_NAME]';

// Used as a scribble pad - don't include trailing slash.
$PHPOF_TMP_PATH = '/tmp';

// E-mail address of sysadmin - used in certain internally-generated error reports 
// You SHOULD set this to a valid e-mail address or local username
if (isset($_SERVER) && isset ($_SERVER['SERVER_ADMIN'])) {
	$PHPOF_ADMIN_EMAIL = $_SERVER['SERVER_ADMIN'];
} else {
	$PHPOF_ADMIN_EMAIL = '';
}

// The database abstraction layer to use: set to 'phpof' for built-in (stable) or
// 'adodb' for (experimental) layer which interfaces with ADODB
$PHPOF_DB_BACKEND = 'phpof';

// The path where ADODB can be found. Only required if PHPOF_DB_BACKEND is set to 'adodb'
// Use trailing slash.
$PHPOF_ADODB_INCLUDE_PATH = '';

// What type of connections to use
// 'connect' = normal
// 'pconnect' = persistent
$PHPOF_DB_CONNECTION_TYPE = 'connect';


/////////////////////////////////////////////////////////////
// MISCELLANEOUS

// Whether to initialise object properties to null in the default constructor
// Turning this on helps to avoid NOTICEs from PHP about undefined properties
$PHPOF_INITIALISE_PROPERTIES_TO_NULL = false;

// Performs a DBRow::clearFields() before DBRow::getDetails();
// Set to false for backward compatibility for snapshot < 20030706
$PHPOF_INITIALISE_PROPERTIES_BEFORE_FETCH = false;


/////////////////////////////////////////////////////////////
// STANDARDS COMPLIANCE

// Whether to enable XHTML-compliant HTML output (e.g. in XArray::CreateSelect)
$PHPOF_XHTML_COMPATIBILITY = true;

// The output character set in use
$PHPOF_OUTPUT_CHARSET = 'ISO-8859-1';


/////////////////////////////////////////////////////////////
// DEBUGGING

// Debugging mode on/off
$PHPOF_DEBUG_MODE = false;

// Whether to dump debug messages to the output stream when debug mode is on
$PHPOF_DEBUG_PRINT = false;

// Whether to log debug messages to a file - set to a filename
// or set to false if you don't want this.
// $PHPOF_DEBUG_LOG = '/path/to/phpof_debug.log';


/////////////////////////////////////////////////////////////
// TABLE CACHING

// Enable table caching? Set to false to disable, otherwise set to a filename
// $PHPOF_TABLE_CACHING = '/tmp/table_cache.db';
$PHPOF_TABLE_CACHING = false;

// DBM-alike database to use for caching. Defaults to gdbm.
// $PHPOF_TABLE_CACHING_TYPE = 'gdbm';


/////////////////////////////////////////////////////////////
// FOREIGN KEYS/RELATIONSHIPS

// How to find foreign key relationships
// You can override this with a call to DBTable::setForeignKeyDiscoveryType()
// Possible values are:
// 'db' : Discover relationships direct from the database (best)
// 'colnames' : Discover relationships by column naming conventions. Can be
// overriden by relations defined in DBTable::ForeignKeys
// 'defined' : Use relations defined in DBTable::ForeignKeys
$PHPOF_FOREIGN_KEY_DISCOVERY_TYPE = 'colnames';

// Allow relations defined in DBTable::ForeignKeys to override discovered ones
$PHPOF_FOREIGN_KEY_DISCOVERY_ALLOW_OVERRIDE = true;


/////////////////////////////////////////////////////////////
// TABLE-CLASS NAME MAPPINGS

// Using this option, you can define mappings of table names and the corresponding
// DBTable class names that you want to be used to create object if/when 
// DB::createTableObject() is called.
// You don't need to define this if you don't use anything that calls DB::createTableObject,
// or if all your table derived classes have names which match your table names exactly.
//$PHPOF_TABLE_CLASS_MAPPINGS = null;


/////////////////////////////////////////////////////////////
// AUDIT LOGGING

// Default on/off (there is a small extra overhead in turning on)
// You can also override this on a per-table basis with the DBTable::AuditLogging property
$PHPOF_AUDIT_LOGGING = false;

// Pre-audit-call hook (optional)
// $PHPOF_HOOK_AUDIT_PRE = '';


/////////////////////////////////////////////////////////////
// PostgreSQL-specific

// Disable DB::insertedID() function? (off by default)
// Improves performance of updates somewhat
$PHPOF_PGSQL_DISABLE_INSERTEDID = false;


/////////////////////////////////////////////////////////////
// MySQL-specific

// Disables discovery of whether MySQL tables are transaction-enabled
$PHPOF_MYSQL_DISABLE_TRANSACTION_CHECKING = false;


/////////////////////////////////////////////////////////////
// ERROR HANDLING

// Use built-in PHPOF error handling? (turn off for testing)
// Set to 'devel' for development
$PHPOF_ERROR_HANDLING = false;

// Throttle errors?
$PHPOF_ERROR_THROTTLE = true;

// Log writes will be throttled to this number of seconds
$PHPOF_ERROR_THROTTLE_LOG = 5;

// E-mails will be throttled to at *least* this number of seconds
// This is the number of seconds since the last *error*, not since 
// the last *e-mail*. So as long as this is larger than the log 
// throttle threshold, you will never see any subsequent e-mails for
// as long as errors are generated more frequently than the e-mail
// throttle threshold.
$PHPOF_ERROR_THROTTLE_EMAIL = 600;

// Use output buffering - flush the buffer and send a std error file
// Set to 'false' if you don't want to use this, or set to a filename.
// $PHPOF_ERROR_PAGE = '500.php';
$PHPOF_ERROR_PAGE = false;

// Show errors?
// Only used if $PHPOF_ERROR_PAGE is false;
$PHPOF_SHOW_ERRORS = true;

// Fail fatally on warnings?
$PHPOF_FAIL_ON_WARNINGS = true;

// E-mail address to send warnings to (if enabled)
$PHPOF_WARNING_EMAIL = $PHPOF_ADMIN_EMAIL;

// E-mail address to send errors to (if enabled)
$PHPOF_ERROR_EMAIL = $PHPOF_ADMIN_EMAIL;

// Log file for warnings
$PHPOF_WARNING_LOG = '';

// Log file for errors
$PHPOF_ERROR_LOG = '';

// What to do with warnings
$PHPOF_LOG_WARNINGS = false;
$PHPOF_MAIL_WARNINGS = true;

// What to do with fatal errors
$PHPOF_LOG_ERRORS = false;
$PHPOF_MAIL_ERRORS = true;

