<?php

// XError.class.php - functions to assist when handling errors. Part of PHPOF.
//
// $Id: XError.class.php 1470 2009-01-11 09:23:19Z tim $
//
// (c)2001-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// DO NOT CHANGE THIS! The config file does not use the constant
define('PHPOF_ERRORHANDLING_DEVEL','devel');

require_once('PHPOF.conf.php');
require_once('XString.class.php');

define('XERROR_FATAL',1);
define('XERROR_WARNING',2);

if (!empty($PHPOF_ERROR_HANDLING)) {
	set_error_handler('HandleError');
	if ($PHPOF_ERROR_PAGE && $PHPOF_ERROR_HANDLING !== 'devel') {
		// We can't really check the value using ob_get_level(), as it could vary
		// according to PHP options set (output_buffering, zlib_output_compression, trans-sid etc.)
		ob_start();
	}
}

// TODO: remove some of the internal duplication here...

/**
 *  This class provides useful functions for dealing with errors and is
 *  primarily intended to be used uninstantiated.
 */
class XError {

	/**
	 * Print out specified errors in an unordered list, using the CSS style 'error'.
	 *
	 * @param array $errors  Errors to print
	 */
	function PrintErrors ($errors) {		
		if ($errors) {
			print "<ul class='error'>\n";
			foreach ($errors as $error) {
				print "<li>$error</li>\n";
			}
			print "</ul>\n";
		}
	}

	// TODO: implement checking of value for error_handling()
	function HandleError ($errno, $errmsg, $filename, $linenum, $vars) {
		global 
			$PHPOF_SITE_NAME, $PHPOF_ERROR_PAGE, $PHPOF_SHOW_ERRORS, $PHPOF_FAIL_ON_WARNINGS, $PHPOF_WARNING_EMAIL, $PHPOF_ERROR_EMAIL, $PHPOF_WARNING_LOG, $PHPOF_ERROR_LOG, $PHPOF_LOG_WARNINGS, $PHPOF_MAIL_WARNINGS, $PHPOF_LOG_ERRORS, $PHPOF_MAIL_ERRORS, $PHPOF_DEBUG_MODE, $PHPOF_ERROR_THROTTLE, $PHPOF_ERROR_THROTTLE_LOG, $PHPOF_ERROR_THROTTLE_EMAIL;

		// Drop out if error is not in error_reporting list
		if  (!(error_reporting() & $errno)) return;

		// Don't waste time; drop out now if it's just a debug notice
		// It would be nice to not receive these at all, but error_reporting() doesn't seem to have an effect.
		$debug_errors = array(E_NOTICE, E_USER_NOTICE);
		if (in_array($errno,$debug_errors)) {
			if ($PHPOF_DEBUG_MODE) phpof_debug("PHP Notice [$filename:$linenum]: $errmsg");
			return;
		}

		// Types of error
    	// in reality the only entries we should
    	// consider are 2,8,256,512 and 1024
		// The others do not result in a call to the error handler.
		$errortype = array (
			E_ERROR => 'Error',
			E_WARNING => 'Warning',
			E_PARSE => 'Parse Error',
			E_NOTICE => 'Notice',
			E_CORE_ERROR => 'Core Error',
			E_CORE_WARNING => 'Core Warning',
			E_COMPILE_ERROR => 'Compile Error',
			E_COMPILE_WARNING => 'Compile Warning',
			E_USER_ERROR => 'User Error',
			E_USER_WARNING => 'User Warning',
			E_USER_NOTICE => 'User Notice',
		);
		if (defined('E_STRICT')) {
			$errortype[E_STRICT] = 'Strict Notice';
		}

		$fatal_errors = array(E_ERROR, E_USER_ERROR);
		$warning_errors = array(E_WARNING, E_USER_WARNING);
		

		$error  = "Date/Time:          ".date("D j M Y H:i:s (T)")."\n";
		$error .= "Site:               $PHPOF_SITE_NAME\n";
		if (isset($_SERVER['HTTP_HOST'])) {
			$error .= "Site Hostname:      ".$_SERVER['HTTP_HOST']."\n";
		}
		$error .= "PHP Error Type:     $errortype[$errno]\n";
		$error .= "PHP Error Details:  $errmsg\n";
		
		$DBError = '';
		if (function_exists('mysql_error')) {
			$DBError = mysql_error();
		}
		if (!empty($DBError)) $error .= "DB Error Details:   $DBError\n";
		
		$RemoteHost = '[Not set; may be a command line script]';
		if (isset($_SERVER['REMOTE_HOST'])) {
			$RemoteHost = $_SERVER['REMOTE_HOST'];
			if (isset($_SERVER['REMOTE_ADDR'])) {
				$RemoteHost .= ' ['.$_SERVER['REMOTE_ADDR'].']';
			}
		} else if (isset($_SERVER['REMOTE_ADDR'])) {
			$RemoteHost = $_SERVER['REMOTE_ADDR'];
		}
		
		$error .= "Script location:    $filename\n";
		$error .= "Line Number:        $linenum\n";
		if (isset($_SERVER['REQUEST_URI'])) {
			$error .= "Request URI:        ".$_SERVER['REQUEST_URI']."\n";
		}
		$error .= "Remote host:        $RemoteHost\n";
		$error .= "\nHTTP parameters:\n";
		
		// HTTP variables
		foreach ($_SERVER as $key => $value) {
			if (substr($key,0,5) == 'HTTP_') {
				$error .= str_pad(substr($key,5).':',20)."$value\n";
			}
		}
		
		$error .= "\n";
		
		if (count($_POST) > 0) {
			$error .= "HTTP POST variables:\n";
			foreach ($_POST as $variable => $value) {
				if (is_array($value)) {
					foreach($value as $array_key => $array_value) {
						// TODO: Iterate this properly...
						$error .= $variable."[$array_key]: $array_value\n";
					}
				} else {
					$error.= "$variable: $value\n";
				}
			}
		}
		
		if (function_exists('debug_backtrace')) {
			$Backtrace = debug_backtrace();
			if (count($Backtrace) > 0) {
				$error .= "\nCall trace:\n";
				$error .= XError::getBacktraceOutput($Backtrace);
			}
		}
		
		if (in_array($errno,$warning_errors)) {
			$Logfile = $PHPOF_WARNING_LOG;
			$Email = $PHPOF_WARNING_EMAIL;
			$SendMail = $PHPOF_MAIL_WARNINGS;
			$WriteLog = $PHPOF_LOG_WARNINGS;
			$Fail = $PHPOF_FAIL_ON_WARNINGS;
		} else {
			$Logfile = $PHPOF_ERROR_LOG;
			$Email = $PHPOF_ERROR_EMAIL;
			$SendMail = $PHPOF_MAIL_ERRORS;
			$WriteLog = $PHPOF_LOG_ERRORS;
			$Fail = true;
		}
		
		$ThrottleError = '';
		if ($Logfile && $PHPOF_ERROR_THROTTLE) {
			$LastErr = @filemtime($Logfile);
			$CurrTime = time();
			
			if ($LastErr > ($CurrTime-$PHPOF_ERROR_THROTTLE_LOG)) {
				$WriteLog = false;
				$SendMail = false;
			} else if ($LastErr > ($CurrTime-$PHPOF_ERROR_THROTTLE_EMAIL)) {
				if ($SendMail == true) {
					$WriteLog = true;
					$ThrottleError = 'E-mail throttled';
				}
				$SendMail = false;
			}
			
			if ($SendMail == true && $WriteLog == false) {
				touch($Logfile);
			}
		}
		
		if ($WriteLog) {
			$ShortError = $RemoteHost . ' ['.date('Y-m-d H:i:s (T)').'] ';
			if (isset($_SERVER['HTTP_HOST']) && isset($_SERVER['REQUEST_URI'])) {
				$ShortError .= $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'].' ';
			}
			$ShortError .= $errmsg . ' ' . $filename . '/' . $linenum;
			if (!empty($DBError)) $ShortError .= " (DB: $DBError)";
			if (!empty($ThrottleError)) $ShortError .= " [$ThrottleError]";
			$ShortError .= "\n";
			XError::_LogError($ShortError, $Logfile);
		}
		if ($SendMail) XError::_MailError($error, $Email);
		
		if ($Fail) {
			if ($PHPOF_ERROR_PAGE) {
				@ob_end_clean();
				include($PHPOF_ERROR_PAGE);
			} elseif ($PHPOF_SHOW_ERRORS) {
				print "\n<pre>$error</pre>\n";
			}
			die();
		}
	}
	
	function _LogError ($error, $filename) {
		global $PHPOF_ADMIN_EMAIL;
		if ($fp = @fopen($filename,'a')) {
			@fputs($fp, $error);
			@fclose($fp);
		} else {
			if ($PHPOF_ADMIN_EMAIL) {
				XError::_MailError("\nCould not write to error log file $filename! Error was:\n\n$error",$PHPOF_ADMIN_EMAIL);
			}
		}
	}
	
	function _MailError ($error, $email) {
		global $PHPOF_SITE_NAME;
		$msg = "$error\n";
		mail($email,'Error on '.$PHPOF_SITE_NAME, $msg);
	}
	
	function Handle404Error () {
		global $PHPOF_ADMIN_EMAIL, $PHPOF_SITE_NAME, $PHPOF_HANDLE_404;
		
		if ($PHPOF_HANDLE_404 !== true) return;
		
		$ignore_list = array(
			'favicon.ico',	// Favicons
			'vti_bin','_vti','cmd.exe','MSOffice','cltreq.asp',	// Worms etc.
			'FMail.cgi','Fmail.cgi','1FM.pl','1fm.pl','1.pl','formMail','Mail.pl','Mail.cgi','form.pl','form.cgi',
			'mail.cgi','fmail.cgi','FMail.pl',
			'formmail','FormMail','Formmail','mail.pl','FORMMAIL','mailform','Form.pl','Form.cgi',	// Spammers
			// Something stupid to do with IE
			'_files','_file','-Dateien','_filer','-filer','_fichiers','_soubory','_archivos','_bestanden','_pliki','.files','_arquivos',
			'mailto'	// Crawlers that can't parse antispam mailto links
		);

		$URL = '[Unknown]';
		if (isset($_SERVER['REQUEST_URI'])) {
			$URL = $_SERVER['REQUEST_URI'];
		}
		$ignore = false;
		foreach ($ignore_list as $ignore_string) {
			if (strstr($URL,$ignore_string)) $ignore = true;
		}
		
		$UserAgent = '';
		if (isset($_SERVER['HTTP_USER_AGENT'])) {
			$UserAgent = $_SERVER['HTTP_USER_AGENT'];
		}
		
		// A broken web crawler running in 138.15.164.0/24
		if ($UserAgent == 'Lachesis' || $UserAgent == 'lachesis') $ignore = true;
		
		// spammer, probably
		if (strstr($UserAgent,'Microsoft URL Control')) $ignore = true;
		if (strstr($UserAgent,'RPT-HTTPClient')) $ignore = true;
		
		// Broken robot of some kind (QuePasa.com?), 69.28.130.230
		if (strstr($UserAgent,'Quepasa')) $ignore = true;
		
		// Some stupid far-east spider which deliberately asks for things in stupid cases
		if (strstr($UserAgent,'Baiduspider')) $ignore = true;
		
		// Almost certainly spamware of some kind
		if (strstr($UserAgent,'Java')) $ignore = true;
		
		// More junk
		if (strstr($UserAgent,'Chilkat')) $ignore = true;
		if (strstr($UserAgent,'Xaldon')) $ignore = true;
		
		if (!$ignore && $PHPOF_ADMIN_EMAIL) {
			$Hostname = gethostbyaddr($IPAddr);
			$msg = 	"There was a 404 error on $PHPOF_SITE_NAME:\r\n\r\n".
						"Requested URI: ".stripslashes($URL)."\r\n";
			if (isset($_SERVER['HTTP_REFERER'])) {
				$msg .= "Referrer:      ".$_SERVER['HTTP_REFERER']."\r\n";
			}
			if (isset($_SERVER['REMOTE_ADDR'])) {
				$IPAddr = $_SERVER['REMOTE_ADDR'];
				$msg .= "Host:          $IPAddr ($Hostname [".gethostbyname($Hostname)."] )"."\r\n";
			}
			$msg .= "User-Agent:    ".$UserAgent;
			
			mail($PHPOF_ADMIN_EMAIL,'404 on '.$PHPOF_SITE_NAME,$msg);
		}
	}
	
	function handleErrorDevel($errno, $errmsg, $filename, $linenum, $vars)
	{
		global $PHPOF_FAIL_ON_WARNINGS;
		
		// Drop out if error is not in error_reporting list
		if  (!(error_reporting() & $errno)) return;
		
		$errortype = array (
			E_ERROR => 'Error',
			E_WARNING => 'Warning',
			E_PARSE => 'Parse Error',
			E_NOTICE => 'Notice',
			E_CORE_ERROR => 'Core Error',
			E_CORE_WARNING => 'Core Warning',
			E_COMPILE_ERROR => 'Compile Error',
			E_COMPILE_WARNING => 'Compile Warning',
			E_USER_ERROR => 'User Error',
			E_USER_WARNING => 'User Warning',
			E_USER_NOTICE => 'User Notice',
		);
		if (defined('E_STRICT')) {
			$errortype[E_STRICT] = 'Strict Notice';
		}
		
		$fatal_errors = array(E_ERROR, E_USER_ERROR);
		$warning_errors = array(E_WARNING, E_USER_WARNING);
		
		// TODO: change output type from html to text depending on php.ini settings
		print "<b>".$errortype[$errno]."</b>: ".XString::OutputHTML($errmsg)." in <b>$filename</b> on line <b>$linenum</b><br />\n";
		
		if (function_exists('mysql_error')) {
			$DBError = mysql_error();
			if (!empty($DBError)) print "<b>DB Error Details</b>: ".XString::OutputHTML($DBError)."<br />\n";
		}
		
		if (phpversion() >= '4.3.0') {
			$Backtrace = debug_backtrace();
			// Backtrace item 1 is the call to XError; item 2 is handlerror;
			// item 3 is the actual file in which is occurred
			// which we've already printed above; so we only want backtrace if more than 3 items
			if (is_array($Backtrace) && count($Backtrace) > 3) {
				print "<pre>Call trace:\n";
				print XError::getBacktraceOutput($Backtrace);
				print "</pre>\n";
			}
		}
		
		if (in_array($errno,$warning_errors)) {
			if ($PHPOF_FAIL_ON_WARNINGS) die();
		} elseif (in_array($errno,$fatal_errors)) {
			die();
		}
	}
	
	function getBacktraceOutput(&$Backtrace)
	{
		$Level = 0;
		$Count = 0;
		$Error = '';
		foreach ($Backtrace as $BacktraceLine) {
			$Count++;
			if ($Count >3) {
				$PrefixHdr = str_repeat('-',$Level*2);
				$Prefix = str_repeat(' ',$Level*2);

				// Not sure under what circumstances these *aren't* set but it certainly happens
				// sometimes in PHP 5.1.4
				$File = (isset($BacktraceLine['file']) ? $BacktraceLine['file'] : '[unknown]');
				$Line = (isset($BacktraceLine['line']) ? $BacktraceLine['line'] : '[unknown]');

				$Error .= $PrefixHdr."Called from file $File\n";
				$Error .= $Prefix." > Line:     $Line\n";
				if (isset($BacktraceLine['type']) && $BacktraceLine['type'] == '->') {
					$Error .= $Prefix." > Function: $BacktraceLine[class]->$BacktraceLine[function]()\n";
				} else {
					if ($BacktraceLine['function']) $Error .= $Prefix." > Function: ".$BacktraceLine['function']."\n";
					if (isset($BacktraceLine['class'])) $Error .= $Prefix." > Class:    ".$BacktraceLine['class']."\n";
					if (isset($BacktraceLine['type'])) $Error .= $Prefix." > Type:     ".$BacktraceLine['type']."\n";
				}
				$Level++;
			}
		}
		return $Error;
	}
}



function HandleError ($errno, $errmsg, $filename, $linenum, $vars) {
	global $PHPOF_ERROR_HANDLING;
	if (isset($PHPOF_ERROR_HANDLING)) {
		if (($PHPOF_ERROR_HANDLING === true || $PHPOF_ERROR_HANDLING === 1) && $PHPOF_ERROR_HANDLING !== PHPOF_ERRORHANDLING_DEVEL) {
			XError::handleError($errno,$errmsg,$filename,$linenum,$vars);
		} else if ($PHPOF_ERROR_HANDLING === PHPOF_ERRORHANDLING_DEVEL) {
			XError::handleErrorDevel($errno,$errmsg,$filename,$linenum,$vars);
		}
	}
}

function phpof_debug ($message) {
	global $PHPOF_DEBUG_MODE,$PHPOF_DEBUG_PRINT,$PHPOF_DEBUG_LOG;
	if ($PHPOF_DEBUG_MODE) {
		if (substr($message, -1) == "\n") $message = substr($message, 0, -1);	// Strip trailing \n
		$Output = date('Y-m-d H:i:s ').'['.$_SERVER['PHP_SELF'].'] '.$message;
		if ($PHPOF_DEBUG_PRINT) print XString::OutputHTML($Output)."<br />\n";
		if ($PHPOF_DEBUG_LOG) {
			if ($fp = @fopen($PHPOF_DEBUG_LOG,'a')) {
				fputs($fp,$Output."\n");
				fclose($fp);
			} else {
				trigger_error("Could not open debug log file $PHPOF_DEBUG_LOG",E_USER_ERROR);
			}
		}
	}
}
