<?php
// adodb.php - ADODB support module for PHPOF
//
// $Id: adodb.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2003 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// Boolean types
define('DB_BOOLEAN_TRUE', true);
define('DB_BOOLEAN_FALSE', false);

function _DBTable_getFields (&$object)
{
	unset($object->_fields);
	$object->_fieldlist = '';	// Initialise rather than unset() to stop E_NOTICE errors when appending
	unset($object->_binaryfields);
	$object->_binaryfieldlist = '';
	
	if (!is_object($object)) trigger_error('PHPOF ADODB _DBTable_getFields() called with non-object',E_USER_ERROR);
	if (!is_object($object->_db)) trigger_error('PHPOF ADODB _DBTable_getFields() called with object without initialised DB',E_USER_ERROR);
	
	
	$Columns = $object->_db->_adodb->MetaColumns($object->_name);
	//$Columns = $object->_db->_adodb->MetaColumns('tests');
	
	//print "<pre>";var_dump($Columns);die();
	
	$PrimaryKeys = $object->_db->_adodb->MetaPrimaryKeys($object->_name);
	//print "<pre>";var_dump($PrimaryKeys);die;
	
	// Could use $Column->primary_key here but doesn't always seem to be defined
	// TODO: store constraints somewhere ($Column->max_length)
	foreach ($Columns as $Column) {
		if ($PrimaryKeys[0] != $Column->name) {
			// TODO: check if binaries work on MSSQL/if this is portable
			if (empty($Column->binary) && $Column->type != 'image') {	// FIXME: binary doesn't seem to be portable in ADODB
				$object->_fields[] = $Column->name;
			} else {
				$object->_binaryfields[] = $Column->name;
			}
			$object->_fieldtype[$Column->name] = $Column->type;
		} else {
			$object->_key = $Column->name;
			
			// FIXME: Balls, we need to find whether column is auto-incrementing
			if (isset($Column->auto_increment)) {
				$object->_key_auto_increment = $Column->auto_increment;
			} else {
				// FIXME: BIG HACK! Assume all primary keys are auto-incrementing
				$object->_key_auto_increment = true;
			}
		}
		
		// TODO: blah, ADODB does not support not_null for MSSQL
		if ($Column->not_null == true) {
			$object->allownull[$Column->name] = false;
		} else {
			$object->allownull[$Column->name] = true;
		}
	}
	
	if (isset($object->_fields)) $object->_fieldlist = implode(',', $object->_fields);
	if (isset($object->_binaryfields))$object->_binaryfieldlist = implode(',', $object->_binaryfields);
	
	// FIXME: Find whether this table supports transactions
	//$object->SupportsTransactions = XXX;
}

// FIXME: don't think this is currently portable across DB's as $Row->_fieldtype will contain types from the DB, not ADODB
// Might be able to use Recordset::MetaType() somehow?
function _DBTable_getColumnInfo (&$Table, &$Column)
{
	//vd($Table->_fieldtype);
	
	if (!empty($Table->_fieldtype[$Column->Name])) {
		
		// Find MySQL data type and constraints
		preg_match("/([a-z]+)(\((.+)\))?/i", $Table->_fieldtype[$Column->Name], $Matches);
		list ($Null,$ColTypeMySQL) = $Matches;
		
		// Constraints
		if (isset($Matches[3])) $Column->Constraints = $Matches[3];
		
		// MySQL type
		$ColTypeMySQL = strtolower($ColTypeMySQL);
		
		// Translate MySQL type to PHPOF type
		switch ($ColTypeMySQL) {
		case 'char':
		case 'varchar':
			$Column->Type = DBTABLE_COLTYPE_STRING;
			break;
		case 'int':
			if ($Column->Constraints == '4') {
				$Column->Type = DBTABLE_COLTYPE_BOOL;	// NB. We are assuming that INT(4) is a bool
			} else {
				$Column->Type = DBTABLE_COLTYPE_INT;
			}
			break;
		case 'smallint':
		case 'mediumint':
		case 'bigint':
			$Column->Type = DBTABLE_COLTYPE_INT;
			break;
		case 'tinytext':
		case 'text':
		case 'mediumtext':
		case 'longtext':
			$Column->Type = DBTABLE_COLTYPE_TEXT;
			break;
		case 'enum':
			$Column->Type = DBTABLE_COLTYPE_ENUM;
			
			// Strip off closing/ending quotes for each item
			$Params = explode(',',$Column->Constraints);
			if (count($Params) > 0) {
				foreach ($Params as $Key => $Value) {
					$Params[$Key] = substr($Value,1,-1);
				}
			}
			$Column->Constraints = implode(',',$Params);
			break;
		case 'set':
			$Column->Type = DBTABLE_COLTYPE_SET;
			break;
		case 'tinyblob':
		case 'blob':
		case 'mediumblob':
		case 'longblob':
			$Column->Type = DBTABLE_COLTYPE_BLOB;
			break;
		case 'float':
		case 'double':
			$Column->Type = DBTABLE_COLTYPE_REAL;
			break;
		default:
			$Column->Type = DBTABLE_COLTYPE_UNKNOWN;
		}
		
		return $Column;
	}
	return false;
}

// TODO: Our foreign key array structure assumes there is only one foreign key per local column
// Is this sensible?
function _DBTable_getForeignKeys_db (&$Table)
{
	$ForeignKeysTranslated = null;
	$ForeignKeys = $this->_adodb->MetaForeignKeys($Table->_name);
	if ($ForeignKeys === false) {
		return PHPOF_NOT_SUPPORTED;
	} else if (is_array($ForeignKeys)) {
		foreach ($ForeignKeys as $ForeignTable => $Keys) {
			foreach ($Keys as $Key) {
				list($LocalCol, $ForeignKey) = explode('=',$Key);
				$ForeignKeysTranslated[$LocalCol] = $ForeignTable.'.'.$ForeignKey;
			}
		}
	}
	return $ForeignKeysTranslated;
}

function _DBRow_updateBinaryData (&$row, $dest_column_name_data, $dest_column_name_type, $filename, $filetype)
{
	$Where = $row->_table->_key.'='.$row->{$row->_table->_key};
	$row->_table->_db->_adodb->UpdateBlobFile($row->_table->_name, $dest_column_name_data, $filename, $Where, 'BLOB');
	$row->_table->_db->query('UPDATE '.$row->_table->_name." SET $dest_column_name_type='$filetype' WHERE ".$row->_table->_key."='".$row->{$row->_table->_key}."'");
}

function _DBRow_updateBinaryDataFromBuffer (&$row, $dest_column_name_data, $dest_column_name_type, &$Data, $filetype)
{
	$Where = $row->_table->_key.'='.$row->{$row->_table->_key};
	$row->_table->_db->_adodb->UpdateBlob($row->_table->_name, $dest_column_name_data, $Data, $Where);
	$row->_table->_db->query('UPDATE '.$row->_table->_name." SET $dest_column_name_type='$filetype' WHERE ".$row->_table->_key."='".$row->{$row->_table->_key}."'");
}

function _DBRow_RetrieveBinaryData (&$row, $column_name, $resultarray)
{
	return $resultarray[$column_name];
}

function _DBRow_RemoveBinaryData (&$row, $column_name)
{
	$row->_table->_db->_adodb->Execute('UPDATE '.$row->_table->_name.
		" SET $column_name=NULL WHERE ".$row->_table->_key."='".$row->{$row->_table->_key}."'");
}

