<?php
// pgsql.php - PostgreSQL support module for PHPOF
//
// $Id: pgsql.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2001-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// Boolean types
define('DB_BOOLEAN_TRUE', 't');
define('DB_BOOLEAN_FALSE', 'f');

function _DBTable_GetFields (&$object) {	
	$object->_fields = array(); // Initialise rather than unset!
	$object->_fieldlist = '';	// Initialise rather than unset() to stop E_NOTICE errors when appending
	unset($object->_binaryfields);
	$object->_binaryfieldlist = '';
	unset($object->_key);
	unset($object->_key_auto_increment);
	
	if (!is_object($object)) trigger_error('PostgreSQL _DBTable_GetFields() called with non-object',E_USER_ERROR);
	if (!is_object($object->_db)) trigger_error('PostgreSQL _DBTable_GetFields() called with object without initialised DB',E_USER_ERROR);

	// This will get column names and types
	// attname === column name
	// format_type === column type
	$resTypes = $object->_db->query(	'SELECT a.attname, format_type(a.atttypid,a.atttypmod) '.
												"FROM pg_class c, pg_attribute a WHERE c.relname='$object->_name' ".
												'AND a.attnum>0 AND a.attrelid=c.oid '.
												'ORDER BY a.attnum');

		
	// This will get any indices for the table
	$resIndexes = $object->_db->query(	'SELECT c2.relname '.
													'FROM pg_class c, pg_class c2, pg_index i '.
													"WHERE c.relname = '$object->_name' AND c.oid = i.indrelid AND i.indexrelid = c2.oid ".
													'ORDER BY c2.relname');
	// This will look for a primary key in the indexes
	unset($index_pkey);
	unset($index_key);
	while (($index = $resIndexes->fetchRow()) && !$index_pkey) {
		if (stristr($index['relname'],'_pkey')) $index_pkey = $index['relname'];
		if (stristr($index['relname'],'_key') && !$index_key) $index_key = $index['relname'];	// Find the FIRST key
	}
	// If there isn't a primary key but we did find a key, assume the key is the pri key
	if (!$index_pkey && $index_key) $index_pkey = $index_key;	
	
	if ($index_pkey) {
		// This will get the index column attributes for the primary key index
		$resKey = 	$object->_db->query(	'SELECT a.attname, a.attnum '.
													'FROM pg_class c, pg_attribute a '.
													"WHERE c.relname = '$index_pkey' ".
													'AND a.attnum > 0 AND a.attrelid = c.oid '.
													'ORDER BY a.attnum');
		if ($resKey->numRows() > 0) {
			$pkey_attributes = $resKey->fetchRow();
		
			// We have found the primary key column name
			$object->_key = $pkey_attributes['attname'];
		
			// Now we need to see if this auto-increments
			// This will get any modifiers for the primary key column, with d.adnum having the column number. Look for "nextval" in the primary key
			$resModifiers = $object->_db->query('SELECT SUBSTRING(d.adsrc FOR 128) AS modifiers '.
															'FROM pg_attrdef d, pg_class c '.
															"WHERE c.relname = '$object->_name' AND c.oid = d.adrelid AND d.adnum=".$pkey_attributes['attnum']);
			if ($resModifiers->numRows() > 0) {
				$modifiers = $resModifiers->fetchRow();
				if (stristr($modifiers['modifiers'], 'nextval')) $object->_key_auto_increment = true;
			}
		}
	}

	while ($field = $resTypes->fetchRow($res)) {				// For each column in the table
		$fieldname = $field['attname'];							// Get the column name
		if ($fieldname != $object->_key) {						// Make sure this is NOT the primary key column
			if (!stristr($field['format_type'],'oid')) {		// If the field is NOT a binary field...
				$object->_fields[] = $fieldname;					// ...then add it to 
				$object->_fieldlist .= "\"$fieldname\",";		// ...the field lists
			} else {														// otherwise
				$object->_binaryfields[] = $fieldname;			// ... add it to the list of binary fields
				$object->_binaryfieldlist .= "\"$fieldname\",";	// ...
			}
			$object->_fieldtype[$fieldname] = $field['format_type'];	// Save the column's datatype
		}
	}
	
	// Strip trailing comma from field lists
	if (isset($object->_fieldlist)) $object->_fieldlist = substr($object->_fieldlist, 0, -1);
	if (isset($object->_binaryfieldlist)) $object->_binaryfieldlist = substr($object->_binaryfieldlist, 0, -1);
	
	// Set flag that table supports transactions
	$object->SupportsTransactions = true;
}

function _DBTable_getColumnInfo (&$Table, &$Column)
{
	// FIXME: Implement PostgreSQL support for this!
	return PHPOF_UNSUPPORTED;
}

// FIXME: Implement PostgreSQL support for this!
function _DBTable_getForeignKeys_db (&$Table)
{
	return PHPOF_NOT_SUPPORTED;
}

function _DBRow_UpdateBinaryData (&$row, $dest_column_name_data, $dest_column_name_type, $filename, $filetype, &$dbh) {
	$row->_table->_db->query(	'UPDATE '.$row->_table->_name.
										" SET $dest_column_name_data = lo_import('$filename'), $dest_column_name_type='$filetype') ".
										'WHERE '.$row->_table->_key."='".$row->{$row->_table->_key}."'");
	$row->$dest_column_name_data = PHPOF_BINARYDATA_PRESENT;
	$row->$dest_column_name_type = $filetype;
}

function _DBRow_updateBinaryDataFromBuffer (&$row, $dest_column_name_data, $dest_column_name_type, &$Data, $filetype, &$dbh) {
	trigger_error('This function is not currently supported with PostgreSQL. To support it, write a suitable function in phpof/dbmodules/pgsql.php:_DBRow_updateBinaryDataFromBuffer()',E_USER_ERROR);
}

function _DBRow_RetrieveBinaryData (&$row, $column_name, $resultarray) {
	global $PHPOF_TMP_PATH;
	// $resultarray is no use to us with Postgres as it will simply contain an OID
	$tempfile = $PHPOF_TMP_PATH.uniqid('phpof');
	$row->_table->_db->query('SELECT lo_export('.$row->_table->_name.".$column_name', '$tempfile') ".
		'FROM '.$row->_table->_name.
		' WHERE '.$row->_table->_key."='".$row->{$row->_table->_key}."'");
	if ($fd = fopen ($tempfile, 'r')) {
		$data = fread($fd,filesize($tempfile));
		fclose($fd);
		unlink($tempfile);
		return $data;
	} else {
		trigger_error('Could not open temp file in pgsql:_DBRow_RetrieveBinaryData',E_USER_ERROR);
	}
	return ;
}

function _DBRow_RemoveBinaryData (&$row, $column_name, &$dbh) {
	$row->_table->_db->query('SELECT lo_unlink('.$row->_table->_name.".$column_name') ".
		'FROM '.$row->_table->_name.
		' WHERE '.$row->_table->_key."='".$row->{$row->_table->_key}."'");
}

